#import "OpenRUMRNBridge.h"
#import <OpenRUM/OpenRUM.h>

@implementation OpenRUMRNBridge

RCT_EXPORT_MODULE(openrumRNBridge);

RCT_EXPORT_METHOD(startWithAppID:(NSString *)appID)
{
    [OpenRUM startWithAppID:appID];
}

RCT_EXPORT_METHOD(setConfigAddress:(NSString *)configAddress)
{
    [OpenRUM setConfigAddress:configAddress];
}

RCT_EXPORT_METHOD(setAppVersion:(NSString *)appVersion)
{
    [OpenRUM setAppVersion:appVersion];
}

RCT_EXPORT_METHOD(setChannelID:(NSString *)channelID)
{
    [OpenRUM setChannelID:channelID];
}

RCT_EXPORT_METHOD(setDeviceID:(NSString *)deviceID)
{
    [OpenRUM setDeviceID:deviceID];
}

RCT_REMAP_METHOD(deviceID,
                 deviceIDWithResolver:(RCTPromiseResolveBlock)resolve rejecter:(RCTPromiseRejectBlock)reject)
{
    NSString *deviceId = [OpenRUM deviceID];
    if (resolve) {
        resolve(deviceId);
    }
}

RCT_REMAP_METHOD(sdkVersion,
                 sdkVersionWithResolver: (RCTPromiseResolveBlock)resolve rejecter:(RCTPromiseRejectBlock)reject)
{
    NSString *sdkVersion = [OpenRUM SDKVersion];
    if (resolve) {
        resolve(sdkVersion);
    }
}

RCT_EXPORT_METHOD(setUserID:(nullable NSString *)userID)
{
   [OpenRUM setUserID:userID];
}

RCT_EXPORT_METHOD(setDropFrameTime:(NSInteger)time)
{
    [OpenRUM setDropFrameTime:time];
}

RCT_EXPORT_METHOD(useCustomLaunch:(BOOL)used)
{
    [OpenRUM useCustomLaunch:used];
}

RCT_EXPORT_METHOD(recordCustomLaunchEnd)
{
    [OpenRUM recordCustomLaunchEnd];
}

/// 自定义异常收集
/// @param exceptionType 异常类型
/// @param causedBy 异常原因
/// @param errorDump 异常堆栈
RCT_EXPORT_METHOD(setCustomException:(NSString *)excType
                  causeBy:(nullable NSString *)causeBy
                  errorDump:(nullable NSString *)errorDump)
{
    [OpenRUM setCustomExceptionWithType:excType causeBy:causeBy errorDump:errorDump];
}

RCT_REMAP_METHOD(setCustomMethodStart,
                 setCustomMethodStartWithName:(NSString *)methodName
                 param:(nullable NSString *)param)
{
    [OpenRUM setCustomMethodStartWithName:methodName
                                     param:param];
}

RCT_REMAP_METHOD(setCustomMethodEnd,
                 setCustomMethodEndWithName:(NSString *)methodName
                 param:(nullable NSString *)param)
{
    [OpenRUM setCustomMethodEndWithName:methodName param:param];
}

#pragma mark - report

/// 点击事件上报
RCT_EXPORT_METHOD(reportAction:(NSString *)time
                  type:(int)type
                  name:(NSString *)name
                  info:(NSString *)info
                  viewName:(NSString *)viewName
                  loadTime:(NSInteger)loadTime)
{
    long long timeValue = [time longLongValue];
   [OpenRUM reportActionWithTime:timeValue
                             type:type
                             name:name
                             info:info
                         viewName:viewName
                         loadTime:loadTime];
}

///视图事件上报
RCT_EXPORT_METHOD(reportView:(NSString *)time
                  viewId:(NSString *)viewId
                  loadTime:(NSInteger)loadTime
                  model:(int)model
                  name:(NSString *)name
                  methodName:(NSString *)methodName)
{
    long long timeValue = [time longLongValue];
    [OpenRUM reportViewWithTime:timeValue
                          viewId:viewId
                        loadTime:loadTime
                           model:model
                            name:name
                      methodName:methodName];
}

/// 崩溃事件上报
RCT_EXPORT_METHOD(reportCrash:(NSString *)time
                  causeby:(NSString *)causeby
                  type:(NSString *)type
                  dump:(NSString *)dump)
{
    long long timeValue = [time longLongValue];
    [OpenRUM reportCrashWithTime:timeValue
                          causeby:causeby
                             type:type
                             dump:dump];
}

@end
