"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ErrorHandler = void 0;
const StringUtils_1 = require("../../util/StringUtils");
function _isReactNativeGlobal(globalScope) {
    return globalScope.ErrorUtils !== undefined;
}
exports.ErrorHandler = (OpenRUMSDK, Logger) => {
    return {
        registerErrorHandler() {
            if (global !== undefined && _isReactNativeGlobal(global)) {
                const oldHandler = global.ErrorUtils.getGlobalHandler();
                global.ErrorUtils.setGlobalHandler((error, isFatal) => {
                    this._reportErrorToOpenRUM(error, isFatal, oldHandler);
                });
                const setter = global.ErrorUtils.setGlobalHandler;
                global.ErrorUtils.setGlobalHandler = (errorHandler) => {
                    setter((error, isFatal) => {
                        this._reportErrorToOpenRUM(error, isFatal, errorHandler);
                    });
                };
            }
        },
        _reportErrorToOpenRUM(exception, isFatal, oldHandler) {
            const evt = (new Date()).getTime();
            if (exception && isExceptionAnError(exception)) {
                if (!StringUtils_1.StringUtils.isStringNullEmptyOrUndefined(exception.name)) {
                    if (isFatal === undefined) {
                        isFatal = false;
                    }
                    if (exception.stack) {
                        OpenRUMSDK.reportErrorFromHandler(evt,isFatal, String(exception.name), exception.message, exception.stack);
                    }
                    else {
                        OpenRUMSDK.reportErrorFromHandler(evt,isFatal,String(exception.name) ,exception.message, null);
                    }
                }
            }
            // else {
            //     OpenRUMSDK.reportError(String(exception), -1);
            // }
            if (oldHandler !== undefined) {
                oldHandler(exception, isFatal);
            }
        }
    };
};
function isExceptionAnError(exception) {
    return exception.message !== undefined;
}
